/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.coverNavigator;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.views.dialogs.CoverNavigatorFrame;
import net.sourceforge.atunes.kernel.controllers.model.FrameController;
import net.sourceforge.atunes.kernel.modules.amazon.GetCoversFromAmazonProcess;
import net.sourceforge.atunes.kernel.modules.process.ProcessListener;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;

import org.jdesktop.swingx.border.DropShadowBorder;

/**
 * The Class CoverNavigatorController.
 */
public class CoverNavigatorController extends FrameController<CoverNavigatorFrame> {

    private static final int COVER_PANEL_WIDTH = Constants.COVER_NAVIGATOR_IMAGE_SIZE + 20;

    private static final int COVER_PANEL_HEIGHT = Constants.COVER_NAVIGATOR_IMAGE_SIZE + 40;

    /**
     * The Class IntermediateResult.
     */
    private static class IntermediateResult {

        /** The album. */
        private Album album;

        /** The cover. */
        private ImageIcon cover;

        /** The grid bag constraints. */
        private GridBagConstraints gridBagConstraints;

        /**
         * Instantiates a new intermediate result.
         * 
         * @param album
         *            the album
         * @param cover
         *            the cover
         * @param gridBagConstraints
         *            the grid bag constraints
         */
        public IntermediateResult(Album album, ImageIcon cover, GridBagConstraints gridBagConstraints) {
            this.album = album;
            this.cover = cover;
            this.gridBagConstraints = gridBagConstraints;
        }

        /**
         * Gets the album.
         * 
         * @return the album
         */
        public Album getAlbum() {
            return album;
        }

        /**
         * Gets the cover.
         * 
         * @return the cover
         */
        public ImageIcon getCover() {
            return cover;
        }

        /**
         * Gets the grid bag constraints.
         * 
         * @return the grid bag constraints
         */
        public GridBagConstraints getGridBagConstraints() {
            return gridBagConstraints;
        }
    }

    /**
     * Instantiates a new cover navigator controller.
     * 
     * @param frame
     *            the frame
     */
    public CoverNavigatorController(CoverNavigatorFrame frame) {
        super(frame);
        addBindings();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
     */
    @Override
    protected void addBindings() {
        final CoverNavigatorFrame frame = getFrameControlled();
        frame.getList().addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (!((JList) e.getSource()).getValueIsAdjusting()) {
                    updateCovers();
                }
            }

        });
        frame.getCoversButton().addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                Artist selectedArtist = (Artist) frame.getList().getSelectedValue();
                if (selectedArtist != null) {
                    GetCoversFromAmazonProcess process = new GetCoversFromAmazonProcess(selectedArtist, getFrameControlled());
                    process.addProcessListener(new ProcessListener() {
                        @Override
                        public void processCanceled() {
                            update();
                        }

                        @Override
                        public void processFinished(boolean ok) {
                            update();
                        }

                        private void update() {
                            try {
                                SwingUtilities.invokeAndWait(new Runnable() {
                                    @Override
                                    public void run() {
                                        updateCovers();
                                    }
                                });
                            } catch (Exception e) {
                                // Nothing to do
                            }
                        }
                    });
                    process.execute();
                }
            }
        });
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
     * ()
     */
    @Override
    protected void addStateBindings() {
        // Nothing to do
    }

    /**
     * Gets the panel for album.
     * 
     * @param album
     *            the album
     * @param cover
     *            the cover
     * @param coversSize
     *            the covers size
     * 
     * @return the panel for album
     */
    JPanel getPanelForAlbum(Album album, ImageIcon cover) {
        JPanel panel = new JPanel(new FlowLayout());

        JLabel coverLabel = new JLabel(cover);
        coverLabel.setToolTipText(album.getName());
        if (cover == null) {
            coverLabel.setPreferredSize(new Dimension(Constants.COVER_NAVIGATOR_IMAGE_SIZE, Constants.COVER_NAVIGATOR_IMAGE_SIZE));
            coverLabel.setBorder(BorderFactory.createLineBorder(GuiUtils.getBorderColor()));
        } else {
            coverLabel.setBorder(new DropShadowBorder());
        }

        JLabel label = new JLabel(album.getName(), SwingConstants.CENTER);
        label.setPreferredSize(new Dimension(Constants.COVER_NAVIGATOR_IMAGE_SIZE, 20));

        panel.add(coverLabel);
        panel.add(label);
        panel.setPreferredSize(new Dimension(COVER_PANEL_WIDTH, COVER_PANEL_HEIGHT));

        GuiUtils.applyComponentOrientation(panel);
        return panel;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
     */
    @Override
    protected void notifyReload() {
        // Nothing to do
    }

    /**
     * Update covers.
     */
    public void updateCovers() {
        final Artist artistSelected = (Artist) getFrameControlled().getList().getSelectedValue();
        if (artistSelected == null) {
            return;
        }

        getFrameControlled().getCoversPanel().removeAll();

        getFrameControlled().getList().setEnabled(false);
        getFrameControlled().getCoversButton().setEnabled(false);
        getFrameControlled().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        SwingWorker<Void, IntermediateResult> generateAndShowAlbumPanels = new SwingWorker<Void, IntermediateResult>() {
            @Override
            protected Void doInBackground() throws Exception {

                final List<Album> albums = new ArrayList<Album>(artistSelected.getAlbums().values());
                Collections.sort(albums);

                int colNumber = getFrameControlled().getCoversScrollPaneWidth() / COVER_PANEL_WIDTH;

                int coversOfLastRow = albums.size() % colNumber;
                if (coversOfLastRow == 0) {
                    coversOfLastRow = colNumber;
                }
                int coversAdded = 0;
                GridBagConstraints c = new GridBagConstraints();
                c.gridx = 0;
                c.gridy = 0;
                for (Album album : albums) {
                    if (coversAdded == albums.size() - coversOfLastRow) {
                        c.weighty = 1;
                        c.fill = GridBagConstraints.BOTH;
                    }
                    if (albums.size() < colNumber && coversAdded == albums.size() - 1) {
                        c.weightx = 1;
                        c.fill = GridBagConstraints.NONE;
                        c.anchor = GridBagConstraints.NORTHWEST;
                    }
                    ImageIcon cover = album.getPicture(Constants.COVER_NAVIGATOR_IMAGE_SIZE, Constants.COVER_NAVIGATOR_IMAGE_SIZE);
                    publish(new IntermediateResult(album, cover, (GridBagConstraints) c.clone()));
                    coversAdded++;
                    c.gridx++;
                    if (c.gridx == colNumber) {
                        c.gridx = 0;
                        c.gridy++;
                    }
                }
                return null;
            }

            @Override
            protected void done() {
                getFrameControlled().setCursor(Cursor.getDefaultCursor());
                getFrameControlled().getList().setEnabled(true);
                getFrameControlled().getCoversButton().setEnabled(true);
            }

            @Override
            protected void process(List<IntermediateResult> intermediateResults) {
                for (IntermediateResult intermediateResult : intermediateResults) {
                    getFrameControlled().getCoversPanel().add(getPanelForAlbum(intermediateResult.getAlbum(), intermediateResult.getCover()),
                            intermediateResult.getGridBagConstraints());
                    getFrameControlled().getCoversPanel().revalidate();
                    getFrameControlled().getCoversPanel().repaint();
                }
            }
        };
        generateAndShowAlbumPanels.execute();
    }

    /**
     * Returns frame controlled
     */
    @Override
    protected CoverNavigatorFrame getFrameControlled() {
        return super.getFrameControlled();
    }

}
