/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.navigation;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.SwingWorker;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.model.NavigationTableColumnModel;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.gui.views.dialogs.AlbumToolTip;
import net.sourceforge.atunes.gui.views.dialogs.SearchDialog;
import net.sourceforge.atunes.gui.views.panels.NavigationPanel;
import net.sourceforge.atunes.gui.views.panels.NavigationFilterPanel.NavigationFilterListener;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.ShowAlbumsInNavigatorAction;
import net.sourceforge.atunes.kernel.actions.ShowArtistsInNavigatorAction;
import net.sourceforge.atunes.kernel.actions.ShowFoldersInNavigatorAction;
import net.sourceforge.atunes.kernel.actions.ShowGenresInNavigatorAction;
import net.sourceforge.atunes.kernel.controllers.model.PanelController;
import net.sourceforge.atunes.kernel.modules.desktop.DesktopHandler;
import net.sourceforge.atunes.kernel.modules.internetsearch.Search;
import net.sourceforge.atunes.kernel.modules.navigator.DeviceNavigationView;
import net.sourceforge.atunes.kernel.modules.navigator.NavigationView;
import net.sourceforge.atunes.kernel.modules.navigator.NavigationViews;
import net.sourceforge.atunes.kernel.modules.navigator.RepositoryNavigationView;
import net.sourceforge.atunes.kernel.modules.repository.RepositoryHandler;
import net.sourceforge.atunes.kernel.modules.repository.RepositoryHandler.SortType;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.misc.log.LogCategories;
import net.sourceforge.atunes.misc.log.Logger;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.model.TreeObject;

/**
 * The Class NavigationController.
 * 
 * @author fleax
 */
public class NavigationController extends PanelController<NavigationPanel> {

    /**
     * The Enum ViewMode.
     */
    public enum ViewMode {

        /** The ARTIST. */
        ARTIST,

        /** The ALBUM. */
        ALBUM,

        /** The GENRE. */
        GENRE,

        /** The FOLDER. */
        FOLDER
    }

    static Logger logger = new Logger();

    /** The current album tool tip content. */
    volatile Object currentAlbumToolTipContent;

    /** The album tool tip. */
    private AlbumToolTip albumToolTip;

    /** The popupmenu caller. */
    private JComponent popupMenuCaller;

    /** The timer. */
    private Timer timer = new Timer(0, new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent arg0) {
            getAlbumToolTip().setVisible(true);

            final Album album = (Album) currentAlbumToolTipContent;
            SwingWorker<ImageIcon, Void> getAndSetImage = new SwingWorker<ImageIcon, Void>() {
                @Override
                protected ImageIcon doInBackground() throws Exception {
                    return album.getPicture(Constants.TOOLTIP_IMAGE_WIDTH, Constants.TOOLTIP_IMAGE_HEIGHT);
                }

                @Override
                protected void done() {
                    try {
                        // Set image in tooltip when done (tooltip can be not visible then)
                        if (currentAlbumToolTipContent != null && currentAlbumToolTipContent.equals(album)) {
                            getAlbumToolTip().setPicture(get());
                        }
                    } catch (InterruptedException e) {
                        logger.error(LogCategories.IMAGE, e);
                    } catch (ExecutionException e) {
                        logger.error(LogCategories.IMAGE, e);
                    }
                }
            };
            getAndSetImage.execute();

        }
    });

    /**
     * Instantiates a new navigation controller.
     * 
     * @param panel
     *            the panel
     */
    public NavigationController(NavigationPanel panel) {
        super(panel);
        addBindings();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
     */
    @Override
    protected void addBindings() {
        getPanelControlled().getNavigationTable().setModel(new NavigationTableModel());
        getPanelControlled().getNavigationTable().setColumnModel(new NavigationTableColumnModel());

        // Add tree selection listeners to all views
        for (NavigationView view : NavigationViews.getNavigationViews()) {
            view.getTree().addTreeSelectionListener(new TreeSelectionListener() {
                @Override
                public void valueChanged(TreeSelectionEvent e) {
                    updateTreeAndTable((JTree) e.getSource());
                }
            });
        }

        // Add tree mouse listeners to all views
        NavigationTreeMouseListener treeMouseListener = new NavigationTreeMouseListener(this);
        for (NavigationView view : NavigationViews.getNavigationViews()) {
            view.getTree().addMouseListener(treeMouseListener);
        }

        NavigationTreeToolTipListener tooltipListener = new NavigationTreeToolTipListener(this);
        NavigationViews.getView(RepositoryNavigationView.class).getTree().addMouseMotionListener(tooltipListener);
        NavigationViews.getView(RepositoryNavigationView.class).getTreeScrollPane().addMouseWheelListener(tooltipListener);

        getPanelControlled().getNavigationTable().addMouseListener(new NavigationTableMouseListener(this, getPanelControlled()));
        getPanelControlled().getTabbedPane().addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                int view = getPanelControlled().getTabbedPane().getSelectedIndex();
                if (view != NavigationViews.indexOf(NavigationViews.getViewByName(ApplicationState.getInstance().getNavigationView()))) {
                    setNavigationView(NavigationViews.getNavigationViews().get(view).getClass().getName());
                }
            }
        });

        getPanelControlled().getTreeFilterPanel().addListener(new NavigationFilterListener() {
            @Override
            public void filterChanged(String newFilter) {
                if (newFilter == null) {
                    getPanelControlled().getTreeFilterPanel().setVisible(false);
                }
                NavigationViews.refreshViews();
            }
        });

        getPanelControlled().getTableFilterPanel().addListener(new NavigationFilterListener() {
            @Override
            public void filterChanged(String newFilter) {
                if (newFilter == null) {
                    getPanelControlled().getTableFilterPanel().setVisible(false);
                }
                updateTreeAndTable(NavigationViews.getCurrentView().getTree());
            }
        });
    }

    @Override
    protected NavigationPanel getPanelControlled() {
        return super.getPanelControlled();
    }

    public NavigationPanel getNavigationPanel() {
        return getPanelControlled();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
     * ()
     */
    @Override
    protected void addStateBindings() {
    }

    /**
     * @return the timer
     */
    public Timer getToolTipTimer() {
        return timer;
    }

    /**
     * Gets the album tool tip.
     * 
     * @return the album tool tip
     */
    public AlbumToolTip getAlbumToolTip() {
        if (albumToolTip == null) {
            JDialog.setDefaultLookAndFeelDecorated(false);
            albumToolTip = new AlbumToolTip();
            JDialog.setDefaultLookAndFeelDecorated(true);
        }
        return albumToolTip;
    }

    /**
     * Get files of all selected elements in navigator.
     * 
     * @return the files selected in navigator
     */
    public List<AudioFile> getFilesSelectedInNavigator() {
        List<AudioObject> files = new ArrayList<AudioObject>();
        if (getPopupMenuCaller() instanceof JTable) {
            int[] rows = getPanelControlled().getNavigationTable().getSelectedRows();
            files.addAll(((NavigationTableModel) getPanelControlled().getNavigationTable().getModel()).getAudioObjectsAt(rows));
        } else if (getPopupMenuCaller() instanceof JTree) {
            TreePath[] paths = NavigationViews.getCurrentView().getTree().getSelectionPaths();
            for (TreePath path : paths) {
                DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) path.getLastPathComponent();
                if (treeNode.getUserObject() instanceof TreeObject) {
                    files.addAll(ControllerProxy.getInstance().getNavigationController().getAudioObjectsForTreeNode(NavigationViews.getCurrentView().getClass(), treeNode));
                }
            }
        }
        return AudioFile.getAudioFiles(files);
    }

    /**
     * Checks if a collection of files have the same parent file.
     * 
     * @param c
     *            collection of files
     * @return if a collection of files have the same parent file
     */
    public boolean sameParentFile(Collection<? extends AudioFile> c) {
        Set<File> set = new HashSet<File>();
        for (AudioFile af : c) {
            set.add(af.getFile().getParentFile());
        }
        return set.size() == 1;
    }

    /**
     * Gets the last album tool tip content.
     * 
     * @return the last album tool tip content
     */
    public Object getCurrentAlbumToolTipContent() {
        return currentAlbumToolTipContent;
    }

    /**
     * Gets the song in navigation table.
     * 
     * @param row
     *            the row
     * 
     * @return the song in navigation table
     */
    public AudioObject getSongInNavigationTable(int row) {
        return ((NavigationTableModel) getPanelControlled().getNavigationTable().getModel()).getSongAt(row);
    }

    public List<AudioObject> getAudioObjectsForTreeNode(Class<? extends NavigationView> navigationViewClass, DefaultMutableTreeNode node) {
        List<AudioObject> audioObjects = NavigationViews.getView(navigationViewClass).getAudioObjectForTreeNode(node, ApplicationState.getInstance().getViewMode(),
                getPanelControlled().getTreeFilterPanel().getFilter());
        if (NavigationViews.getView(navigationViewClass).isAudioObjectsFromNodeNeedSort()) {
            return RepositoryHandler.getInstance().sort(audioObjects, ApplicationState.getInstance().getSortType());
        }
        return audioObjects;
    }

    /**
     * Gets the songs for navigation tree.
     * 
     * @return the songs for navigation tree
     */
    public List<AudioObject> getSongsForNavigationTree() {
        TreePath[] paths = NavigationViews.getView(RepositoryNavigationView.class).getTree().getSelectionPaths();
        if (paths != null) {
            List<AudioObject> audioObjects = new ArrayList<AudioObject>();
            for (TreePath element : paths) {
                audioObjects.addAll(getAudioObjectsForTreeNode(RepositoryNavigationView.class, (DefaultMutableTreeNode) element.getLastPathComponent()));
            }
            return audioObjects;
        }
        return null;
    }

    /**
     * Notify device reload.
     */
    public void notifyDeviceReload() {
        NavigationViews.getView(DeviceNavigationView.class).refreshView(ApplicationState.getInstance().getViewMode(), getPanelControlled().getTreeFilterPanel().getFilter());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
     */
    @Override
    public void notifyReload() {
        NavigationViews.refreshViews();
    }

    /**
     * Refresh table.
     */
    public void refreshTable() {
        ((NavigationTableModel) getPanelControlled().getNavigationTable().getModel()).refresh();
    }

    /**
     * Sets the current album tool tip content.
     * 
     * @param currentAlbumToolTipContent
     *            the new current album tool tip content
     */
    public void setCurrentAlbumToolTipContent(Object currentAlbumToolTipContent) {
        this.currentAlbumToolTipContent = currentAlbumToolTipContent;
    }

    /**
     * Sets the navigation view.
     * 
     * @param view
     *            the new navigation view
     */
    public void setNavigationView(String view) {
        getLogger().debug(LogCategories.CONTROLLER, new String[] { view });

        Class<? extends NavigationView> navigationView = NavigationViews.getViewByName(view);
        if (navigationView == null) {
            navigationView = RepositoryNavigationView.class;
        }
        ApplicationState.getInstance().setNavigationView(navigationView.getName());

        int currentView = getPanelControlled().getTabbedPane().getSelectedIndex();
        int newView = NavigationViews.indexOf(navigationView);
        // If current view is equals to the new view then don't change tabbed pane selected index
        // This can happen when this method is called from a stateChanged method of tabbed pane listener when user
        // changes selected tab
        if (currentView != newView) {
            getPanelControlled().getTabbedPane().setSelectedIndex(NavigationViews.indexOf(navigationView));
        }

        boolean viewModeSupported = NavigationViews.getView(navigationView).isViewModeSupported();
        Actions.getAction(ShowAlbumsInNavigatorAction.class).setEnabled(viewModeSupported);
        Actions.getAction(ShowArtistsInNavigatorAction.class).setEnabled(viewModeSupported);
        Actions.getAction(ShowFoldersInNavigatorAction.class).setEnabled(viewModeSupported);
        Actions.getAction(ShowGenresInNavigatorAction.class).setEnabled(viewModeSupported);

        // Clear tree filter
        getPanelControlled().getTreeFilterPanel().setFilter(null);
        getPanelControlled().getTreeFilterPanel().setVisible(false);
        NavigationViews.refreshViews();

        // Clear table filter
        getPanelControlled().getTableFilterPanel().setFilter(null);
        getPanelControlled().getTableFilterPanel().setVisible(false);

        JTree tree = NavigationViews.getCurrentView().getTree();

        if (tree.getSelectionPath() != null) {
            ((NavigationTableModel) getPanelControlled().getNavigationTable().getModel()).setSongs(getAudioObjectsForTreeNode(navigationView, (DefaultMutableTreeNode) (tree
                    .getSelectionPath().getLastPathComponent())));
        }        
    }

    /**
     * Sort.
     * 
     * @param songs
     *            the songs
     * @param type
     *            the type
     * 
     * @return the list< audio object>
     */
    public List<AudioObject> sort(List<AudioObject> songs, SortType type) {
        return RepositoryHandler.getInstance().sort(songs, type);
    }

    /**
     * Tree selection.
     * 
     * @param tree
     *            the tree
     */
    public void updateTreeAndTable(JTree tree) {
        List<AudioObject> songs = new ArrayList<AudioObject>();
        TreePath[] paths = tree.getSelectionPaths();

        // Avoid events when changes on a tree different than the one which is visible
        if (tree != NavigationViews.getCurrentView().getTree()) {
            return;
        }

        if (paths != null) {
            for (TreePath element : paths) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) (element.getLastPathComponent());
                songs.addAll(getAudioObjectsForTreeNode(NavigationViews.getViewByName(ApplicationState.getInstance().getNavigationView()), node));
            }

            songs = filterNavigationTable(songs);

            ((NavigationTableModel) getPanelControlled().getNavigationTable().getModel()).setSongs(songs);
        }
    }

    /**
     * Returns a filtered list of audio objects using the current table filter
     * 
     * @param audioObjects
     * @return
     */
    private List<AudioObject> filterNavigationTable(List<AudioObject> audioObjects) {
        if (getPanelControlled().getTableFilterPanel().getFilter() == null) {
            return audioObjects;
        }

        if (!NavigationViews.getCurrentView().isNavigatorTableFilterSupported()) {
            return audioObjects;
        }

        return NavigationViews.getCurrentView().filterNavigatorTable(audioObjects, getPanelControlled().getTableFilterPanel().getFilter());
    }

    /**
     * Open search dialog.
     * 
     * @param dialog
     *            the dialog
     * @param setAsDefaultVisible
     *            the set as default visible
     * 
     * @return the search
     */
    public Search openSearchDialog(SearchDialog dialog, boolean setAsDefaultVisible) {
        dialog.setSetAsDefaultVisible(setAsDefaultVisible);
        dialog.setVisible(true);
        Search s = dialog.getResult();
        if (getPopupMenuCaller() == NavigationViews.getView(RepositoryNavigationView.class).getTree()) {
            TreePath path = NavigationViews.getView(RepositoryNavigationView.class).getTree().getSelectionPath();
            if (((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject() instanceof Artist) {
                Artist a = (Artist) ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject();
                DesktopHandler.getInstance().openSearch(s, a.getName());
            }
        }
        return s;
    }

    /**
     * @return the popupMenuCaller
     */
    public JComponent getPopupMenuCaller() {
        return popupMenuCaller;
    }

    /**
     * @param popupMenuCaller
     *            the popupMenuCaller to set
     */
    public void setPopupMenuCaller(JComponent popupMenuCaller) {
        this.popupMenuCaller = popupMenuCaller;
    }
}
