/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.pattern;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.EditTagInfo;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

/**
 * Defines patterns based on AudioFile properties to get strings to be used
 * in file or folder names
 * 
 * @author fleax
 * 
 */
public abstract class Pattern {

    private static final char PATTERN_NAME_FIRST_CHAR = '%'; 
    
    /**
     * All available patterns
     */
    private static List<Pattern> patterns;
    
    /**
     * DUMMY PATTERN, USED TO MATCH ANYTHING
     */
    private static final Pattern ANY_PATTERN = new Pattern('?', "ANY", true, true) {
        @Override
        public String getAudioFileStringValue(AudioFile audioFile) {
            // This is a dummy pattern
            return null;
        }
    };
    
    /**
     * Returns all available patterns
     * @return
     */
    public static List<Pattern> getPatterns() {
        if (patterns == null) {
            patterns = new ArrayList<Pattern>();
            
            // DUMMY PATTERN, USED TO MATCH ANYTHING
            patterns.add(ANY_PATTERN);
            
            patterns.add(new Pattern('T', "TITLE", true, false) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    return audioFile.getTitleOrFileName();
                }
            });
            patterns.add(new Pattern('A', "ARTIST", true, true) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    return audioFile.getArtist();
                }
            });
            patterns.add(new Pattern('L', "ALBUM", true, true) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    return audioFile.getAlbum();
                }
            });
            patterns.add(new Pattern('R', "ALBUM_ARTIST", true, true) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    return audioFile.getAlbumArtist();
                }
            });
            patterns.add(new Pattern('N', "TRACK", true, false) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    String track = audioFile.getTrackNumber().toString();
                    return track.length() < 2 ? StringUtils.getString("0", track) : track;
                }
            });
            patterns.add(new Pattern('G', "GENRE", true, true) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    return audioFile.getGenre();
                }
            });
            patterns.add(new Pattern('Y', "YEAR", true, true) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    return audioFile.getYear();
                }
            });
            patterns.add(new Pattern('C', "COMPOSER", true, true) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    return audioFile.getComposer();
                }
            });
            patterns.add(new Pattern('S', "ARTIST_FIRST_CHAR", false, false) {
                @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                    return audioFile.getArtist().substring(0, 1);
                }
            });
            patterns.add(new Pattern('D', "DISC_NUMBER", true, true) {
               @Override
                public String getAudioFileStringValue(AudioFile audioFile) {
                   return audioFile.getDiscNumber().toString();
                } 
            });
        }
        return patterns;
    }
    
    /**
     * Return only patterns used for massive recognition
     * @return
     */
    public static List<Pattern> getMassiveRecognitionPatterns() {
        List<Pattern> result = new ArrayList<Pattern>();
        for (Pattern p : getPatterns()) {
            if (p.massiveRecognitionPattern) {
                result.add(p);
            }
        }
        return result;
    }

    /**
     * Return only patterns used for recognition (both massive or not)
     * @return
     */
    public static List<Pattern> getRecognitionPatterns() {
        List<Pattern> result = new ArrayList<Pattern>();
        for (Pattern p : getPatterns()) {
            if (p.recognitionPattern) {
                result.add(p);
            }
        }
        return result;
    }

    /**
     * Chars representing this pattern
     */
    private String pattern;
    
    /**
     * Name of this pattern
     */
    private String name;
    
    /**
     * <code>true</code> if this pattern can be used for massive pattern recognition
     * 
     * For example: song title or track number are not massive patterns as it only apply to one song at a time
     *  
     */
    boolean massiveRecognitionPattern = false;
    
    /**
     * <code>true</code> if this pattern can be used for pattern recognition
     * 
     * For example: To be used for recognition application must be able to assign a full attribute value from a pattern
     *              Fist artist char is not a recognition pattern as it is not a complete attribute of a song
     *               
     */
    boolean recognitionPattern = false;
    
    /**
     * Private constructor
     * @param patternChar
     * @param name
     * @param recognitionPattern
     * @param massiveRecognitionPattern
     */
    protected Pattern(char patternChar, String name, boolean recognitionPattern, boolean massiveRecognitionPattern) {
        // Force upper case
        this.pattern = StringUtils.getString(PATTERN_NAME_FIRST_CHAR, patternChar).toUpperCase();
        this.name = name;
        this.recognitionPattern = recognitionPattern;
        this.massiveRecognitionPattern = massiveRecognitionPattern;
    }

    /**
     * @return the pattern
     */
    public String getPattern() {
        return pattern;
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * @return the description of this pattern
     */
    public String getDescription() {
        return LanguageTool.getString(name);
    }
    
    /**
     * Returns a string, result of apply this pattern to an audio file object
     * 
     * @param audioFile
     * @return
     */
    public String applyPattern(String sourceString, AudioFile audioFile) {
        if (!pattern.equals(ANY_PATTERN.pattern)) {
            return sourceString.replaceAll(pattern, getAudioFileStringValue(audioFile));
        }
        return sourceString;
    }

    /**
     * Returns string value of an AudioFile to do transformation
     * 
     * @param audioFile
     * @return
     */
    public abstract String getAudioFileStringValue(AudioFile audioFile);
    
    /**
     * Returns a String as result of applying all defined patterns over a pattern string and a AudioFile
     * @param pattern
     * @param song
     * @return
     */
    public static String applyPatternTransformations(String pattern, AudioFile song) {
        String result = pattern;
        for (Pattern transform : getPatterns()) {
            result = transform.applyPattern(result, song);
        }
        return result;
    }

    /**
     * Returns a EditTagInfo object from a map of matches. Every attribute is filled with the matched value if found in map
     * @param matches
     * @return
     */
    public static EditTagInfo getEditTagInfoFromMatches(Map<String, String> matches) {
        EditTagInfo tagInfo = new EditTagInfo();
        for (String match : matches.keySet()) {
            tagInfo.put(match, matches.get(match));
        }
        return tagInfo;        
    }

    /**
     * Returns a map containing string values result of matching a pattern against a string
     * @param patterns
     * @param value
     * @param onlyMassiveRecognitionPatterns
     * @return
     */
    public static Map<String, String> getPatternMatches(String patterns, String value, boolean onlyMassiveRecognitionPatterns) {
        Map<String, String> matches = new HashMap<String, String>();
        
        if (patterns == null || value == null) {
            return matches;
        }
        
        String patternsString = patterns.trim();
        
        // Get all non-pattern sequences
        String[] nonPatternSequences = patternsString.split(StringUtils.getString(PATTERN_NAME_FIRST_CHAR, '.'));

        int valueIndex = 0;
        int patternsStringIndex = 0;
        
        int i = 0;
        String matchedPattern = null;
        String matchedValue = null;
        
        // Start parsing patterns string
        while (patternsStringIndex < patternsString.length() && valueIndex < value.length()) {
            // Parsing before last non pattern sequence
            if (i < nonPatternSequences.length) {
                if (nonPatternSequences[i].isEmpty()) {
                    i++;
                    continue;
                }
                
                // Get index of current non pattern sequence
                int indexAtPatternsString = patternsString.indexOf(nonPatternSequences[i], patternsStringIndex);
                int indexAtValueString = value.indexOf(nonPatternSequences[i], valueIndex);
                
                // End of patterns string
                if (indexAtPatternsString == -1) {
                    break;
                }
                
                // If value string ended without finding non pattern sequence, then the next pattern is substring
                // from valueIndex to end of string
                // NOTE this is a less restricted pattern search, as means that the last patterns in a pattern string
                // are optional
                if (indexAtValueString == -1) {
                    indexAtValueString = value.length();
                }
                
                // We found a matched pattern
                matchedPattern = patternsString.substring(patternsStringIndex, indexAtPatternsString);
                matchedValue = value.substring(valueIndex, indexAtValueString);
                
                // Update indexes
                patternsStringIndex = patternsStringIndex + matchedPattern.length() + nonPatternSequences[i].length();
                valueIndex = valueIndex + matchedValue.length() + nonPatternSequences[i].length();
                
                i++;                
            } else {
                // After last non pattern sequence we have a new matched pattern until end of string
                matchedPattern = patternsString.substring(patternsStringIndex);
                matchedValue = value.substring(valueIndex);
                
                patternsStringIndex = patternsStringIndex + matchedPattern.length();
            }
            
            // Force upper case patterns
            matchedPattern = matchedPattern.toUpperCase();
            
            // Duplicate patterns are not allowed (despite being ? pattern) so we return an empty map
            if (!matchedPattern.equals(ANY_PATTERN.getPattern()) && matches.containsKey(matchedPattern)) {
                return new HashMap<String, String>();
            }
            
            // Ignore ? pattern
            if (!matchedPattern.equals(ANY_PATTERN.getPattern())) {
                matches.put(matchedPattern, matchedValue.trim());
            }
        }

        // Now return only necessary patterns
        Map<String, String> result = new HashMap<String, String>();
        List<Pattern> patternsToBeUsed = onlyMassiveRecognitionPatterns ? getMassiveRecognitionPatterns() : getPatterns();
        for (Pattern p : patternsToBeUsed) {
            if (matches.containsKey(p.getPattern())) {
                result.put(p.getName(), matches.get(p.getPattern()));
            }
        }
        
        return result;
    }    
}
